#include "HDC1000_Defs.h"

sbit sRDY at GPIO_PIN3_bit;
char tmp_data[12];

float temperature;
float humidity;
float dewpoint;

static void sensor_setup() {
  tmp_data[0] = 0x02;
  tmp_data[1] = 0x10;
  tmp_data[2] = 0x00;
  I2CM_Set_Slave_Address(0x40);
  I2CM1_Write_Bytes(&tmp_data, 3);
  delay_ms(5);
}

//Conversion time for 14bit values of temperature and humidity 6,5ms + 6,35ms
static void wait_conversion() {
  delay_ms(20);
}

volatile unsigned int temp_value,humidity_value;

void Read_Sensor() {
  tmp_data[0] = TEMP_I2C_ADDR;
  I2CM_Set_Slave_Address(0x40);
  I2CM1_Write_Bytes(&tmp_data, 1);
  wait_conversion();
  I2CM_Set_Slave_Address(HDC1000_I2C_ADDR);
  I2CM1_Read_Bytes(&tmp_data, 4);
  temp_value = (tmp_data[0] << 8) + tmp_data[1];
  humidity_value = (tmp_data[2] << 8) + tmp_data[3];
  temperature = ((float)temp_value * Tconstant)-40;
  humidity = (float)humidity_value * Hconstant;
}

void Read_Temperature() {
  unsigned int temp_value;
  tmp_data[0] = TEMP_I2C_ADDR;
  I2CM_Set_Slave_Address(0x40);
  I2CM1_Write_Bytes(&tmp_data, 1);
  delay_ms(7);
  I2CM_Set_Slave_Address(HDC1000_I2C_ADDR);
  I2CM1_Read_Bytes(&tmp_data, 2);
  temp_value = (tmp_data[0] << 8) + tmp_data[1];
  temperature = ((float)temp_value * Tconstant)-40;
}

void Read_Humidity() {
  unsigned int humidity_value;
  tmp_data[0] = TEMP_I2C_ADDR;
  I2CM_Set_Slave_Address(0x40);
  I2CM1_Write_Bytes(&tmp_data, 1);
  delay_ms(7);
  I2CM_Set_Slave_Address(HDC1000_I2C_ADDR);
  I2CM1_Read_Bytes(&tmp_data, 2);
  humidity_value = (tmp_data[2] << 8) + tmp_data[3];
  humidity = (float)humidity_value * Hconstant;
}

unsigned int Read_DeviceID() {
  unsigned int id_value;
  tmp_data[0] = DEVICEID_I2C_ADDR;
  I2CM_Set_Slave_Address(0x40);
  I2CM1_Write_Bytes(&tmp_data, 1);
  I2CM_Set_Slave_Address(HDC1000_I2C_ADDR);
  I2CM1_Read_Bytes(&tmp_data, 2);
  id_value = (tmp_data[0] << 8) + tmp_data[1];
  return id_value;
}

char HDC1000_Init() {
  GPIO_Pin_Digital_Input(45);
  GPIO_Pin_Digital_Output(44);
  //I2C INITIALIZATION GPIO44=SCL;  GPIO45=SDA
  //If bus is busy wait SDA high before initializing I2C module
  GPIO_PIN44_bit = 1;
  while (GPIO_PIN45_bit == 0) {
    GPIO_PIN44_bit = 0;
    Delay_us(10);
    GPIO_PIN44_bit = 1;
    Delay_us(10);
  }
  I2CM1_Init(_I2CM_SPEED_MODE_FAST, _I2CM_SWAP_DISABLE);
  delay_ms(200);
  sensor_setup();
  return 1;
}

//August-Roche-Magnus approximation.
//http://en.wikipedia.org/wiki/Clausius%E2%80%93Clapeyron_relation#Meteorology_and_climatology
//tested: http://andrew.rsmas.miami.edu/bmcnoldy/Humidity.html
void Calculate_Dewpoint() {
  dewpoint = 243.04*(log(humidity/100)+((17.625*temperature)/(243.04+temperature)))
            /(17.625-log(humidity/100)-((17.625*temperature)/(243.04+temperature)));
}