/*
 * Project name:
     HDC1000 click (Demonstration of mikroBUS HDC1000 click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20150326:
       - initial release (DM);
 * Description:
     This code demonstrates how to use HDC1000 click with EasyFT90x Pro v7 board.
     HDC1000 sensor measures temperature and relative humidity.
 * Test configuration:
     Device:          FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev.Board:       EasyFT90x v7
                      http://www.mikroe.com/easyft90x/
     Oscillator:      CPU at full system clock, 100.000MHz
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - Place HDC1000 click board at the mikroBUS socket 1 on the EasyFT90x v7 board.
     - Put EasyTFT in its socket, turn on backlight using the SW5.8
 */
 
#include "HDC1000_Defs.h"
#include "HDC1000_Driver.h"
#include "resources.h"

char txt[16];
extern float temperature;
extern float humidity;
extern float dewpoint;
unsigned int device_id;

// TFT module connections
char TFT_DataPort at GPIO_PORT_32_39;
sbit TFT_RST at GPIO_PIN42_bit;
sbit TFT_RS at GPIO_PIN40_bit;
sbit TFT_CS at GPIO_PIN41_bit;
sbit TFT_RD at GPIO_PIN46_bit;
sbit TFT_WR at GPIO_PIN47_bit;
sbit TFT_BLED at GPIO_PIN58_bit;
// End TFT module connections

void Auxiliary_Code() {
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Set_Index(unsigned short index) {
  TFT_RS = 0;
  asm nop;
  TFT_DataPort = index;
  asm nop; asm nop;
  Auxiliary_Code();
}

void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  asm nop;
  TFT_DataPort = cmd;
  asm nop; asm nop;
  Auxiliary_Code();
}

void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  asm nop;
  TFT_DataPort = (_data >> 8);
  asm nop; asm nop;
  Auxiliary_Code();
  asm nop;
  TFT_DataPort = (_data);
  asm nop; asm nop;
  Auxiliary_Code();
}

void Display_Init(){
  TFT_Set_Active(Set_Index, Write_Command, Write_Data);
  TFT_Init_ILI9341_8bit(320, 240);
  TFT_BLED = 1;
  TFT_Set_Pen(CL_WHITE, 1);
  TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
  TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_Black, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("HDC1000 Click", 85, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyFT90x v7", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Image_Jpeg(210, 75, rain_cloud_glossy_jpg);
}

void Draw_screen() {
  TFT_Set_Font(&Tahoma15x16_Bold, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Temperature:",20,70);
  TFT_Write_Text("Relative Humidity:",20,100);
  TFT_Write_Text("Dewpoint:",20,130);
}

void Display_Values() {
  TFT_Set_Pen(CL_WHITE, 1);
  //temperature
  sprintf(txt, "%2.1f C", temperature);
  TFT_Rectangle(115,70,210,90);
  TFT_Write_Text(txt,115,70);
  //humidity
  sprintf(txt, "%2.1f", humidity);
  TFT_Rectangle(140,100,210,120);
  TFT_Write_Text(txt,140,100);
  TFT_Write_Char(0x25,170,100);
  //dewpoint
  sprintf(txt, "%2.1f C", dewpoint);
  TFT_Rectangle(90,130,210,150);
  TFT_Write_Text(txt,90,130);
}

void main() {
  Display_Init();
  while(!HDC1000_Init()); //wait for initialization
  device_id = Read_DeviceID();
  if (device_id = ID_NUMBER) {
    Draw_Screen();
    while(1) {
      Read_Sensor();
      Calculate_Dewpoint();
      Display_Values();
      delay_ms(2000);
    }
  }

  else {
    TFT_Write_Text("Error",10,30);
    while(1);
  }
}